@@ -31,6 +31,7 @@ def send(message, to="*", exclude=()):
 
 unit_states = ["idle", "walking", "frozen", "throwing", "gathering", "entering",
         "inbuilding", "unloading", "getting_snowball", "stray"]
+holding_types = [None, "fish", "crystal"]
 
 
 class Connection(object):
@@ -266,14 +267,10 @@ class Message(object):
 
 class MUnitPosition(Message):
     type_id = 100
-    struct_format = "!HhhbbB"
-    attrs = ('unit_id', 'x', 'y', 'dir_x', 'dir_y', 'state_id')
+    struct_format = "!HhhbbBB"
+    attrs = ('unit_id', 'x', 'y', 'dir_x', 'dir_y', 'state_id', 'holding_id')
 
-
-class MUnitHolding(Message):
-    type_id = 101
-    struct_format = "!HB"
-    attrs = ("unit_id", "holding")
+# 101 is free
 
 class MUnitAddEmblem(Message):
     type_id = 102
@@ -283,13 +280,13 @@ class MUnitAddEmblem(Message):
         w.single("H", self.unit_id)
         w.string(self.image_name)
         w.single("B", self.animate)
-        w.single("HH", *self.offset)
+        w.single("hh", *self.offset)
         return w.data.getvalue()
     @classmethod
     def from_stream(cls, stream):
         r = Reader(stream)
         return cls(r.single("H")[0], r.string(), bool(r.single("B")),
-                r.single("HH"))
+                r.single("hh"))
 
 class MUnitRemoveEmblem(Message):
     type_id = 103
@@ -326,6 +323,11 @@ class MBuildingJeopardy(Message):
     struct_format = "!HB"
     attrs = ('building_id', 'jeopardy')
 
+class MBuildingStorage(Message):
+    type_id = 112
+    struct_format = "!Hhh"
+    attrs = ("building_id", 'fish', 'crystal')
+
 
 class MCreateDynamicMapObject(Message):
     type_id = 114
@@ -361,6 +363,12 @@ class MDMOPosition(Message):
     # TODO
 
 
+class MResourceQuantity(Message):
+    type_id = 117
+    struct_format = "!hhh"
+    attrs = ("tx", "ty", "q")
+
+
 # Player/connection stuff
 
 class MNewPlayer(Message):
@@ -452,11 +460,24 @@ class MPlayerVictory(Message):
         return cls(r.single("H")[0],
                 dict(t=True, f=False, n=None)[r.single("s")[0]])
 
-message_classes = [MUnitPosition, MBuildingOwner, MBuildingJeopardy, MSnowball,
-        MWarmth, MNewPlayer, MWhoYouAre, MSetJob, MDisbanUnits,
-        MCreateDynamicMapObject, MDMOHidden, MDMOPosition, MUnitAddEmblem,
-        MUnitRemoveEmblem, MUnitHolding, MGameStart, MChatMessage,
-        MUnitIglooChange, MClientFinishedLoading, MPlayerVictory]
+class MMapname(Message):
+    type_id = 227
+    attrs = ("mapname",)
+    def pack(self):
+        w = Writer()
+        w.string(self.mapname)
+        return w.data.getvalue()
+    @classmethod
+    def from_stream(cls, stream):
+        r = Reader(stream)
+        return cls(r.string())
+
+message_classes = [MUnitPosition, MBuildingOwner, MBuildingJeopardy,
+        MBuildingStorage, MSnowball, MWarmth, MNewPlayer, MWhoYouAre, MSetJob,
+        MDisbanUnits, MCreateDynamicMapObject, MDMOHidden, MDMOPosition,
+        MResourceQuantity, MUnitAddEmblem, MUnitRemoveEmblem, MGameStart,
+        MChatMessage, MUnitIglooChange, MClientFinishedLoading, MPlayerVictory,
+        MMapname]
 
 indexed_message_classes = {}
 for cls in message_classes:
@@ -495,12 +516,6 @@ class MasterUnit(Unit):
         #self.call_remote("update_max_warmth", mw)
     #max_warmth = property(lambda self:self._max_warmth, _set_max_warmth)
 
-    def _set_holding(self, h):
-        self._holding = h
-        if hasattr(self, "unit_id"):
-            send(MUnitHolding(self.unit_id, bool(h)))
-    holding = property((lambda self:self._holding), _set_holding)
-
     def _set_igloo(self, i):
         self._igloo = i
         if hasattr(self, "unit_id") and hasattr(i, "building_id"):
@@ -514,7 +529,8 @@ class MasterUnit(Unit):
                     self.state not in ['throwing', 'getting_snowball']):
             send(MUnitPosition(self.unit_id, self.x, self.y,
                     self.direction[0], self.direction[1],
-                    unit_states.index(self.state)))
+                    unit_states.index(self.state),
+                    holding_types.index(self.holding)))
         self._last_state_sent = self.state
 
     def throw_snowball(self, target):
@@ -525,10 +541,9 @@ class MasterUnit(Unit):
         return s
 
     def add_emblem(self, emblem):
-        if not emblem in self._emblems:
-            m = MUnitAddEmblem(self.unit_id, emblem.image_name, emblem.animate,
-                    emblem.offset)
-            send(m)
+        m = MUnitAddEmblem(self.unit_id, emblem.image_name, emblem.animate,
+                emblem.offset)
+        send(m)
         Unit.add_emblem(self, emblem)
 
     def remove_emblem(self, emblem):
@@ -546,13 +561,14 @@ class SlaveUnit(Unit):
 
     def handle_MUnitPosition(self, m):
         state = unit_states[m.state_id]
-        self._next_position = ((m.x, m.y), (m.dir_x, m.dir_y), state)
+        holding = holding_types[m.holding_id]
+        self._next_position = ((m.x, m.y), (m.dir_x, m.dir_y), state, holding)
 
     def handle_MSnowball(self, m):
         self.snowballs = m.snowballs_left
         #self.current_frame = 1
         dir = self.direction_to_tile(m.x//32, m.y//32)
-        self._next_position = ((m.px, m.py), dir, "throwing")
+        self._next_position = ((m.px, m.py), dir, "throwing", None)
         self._next_snowball = (m.x, m.y)
 
     def handle_MWarmth(self, m):
@@ -564,12 +580,6 @@ class SlaveUnit(Unit):
         except KeyError:
             pass
 
-    def handle_MUnitHolding(self, m):
-        if m.holding:
-            self.holding = "fish" # TODO other types
-        else:
-            self.holding = None
-
     def handle_MUnitAddEmblem(self, m):
         self.add_emblem(Emblem(m.image_name, m.animate, m.offset))
 
@@ -578,6 +588,7 @@ class SlaveUnit(Unit):
             self.remove_emblem(e)
 
     def move(self):
+        startxy = self.x, self.y
         try_next_position = True
         if self.state == "throwing" and self.current_frame < 9:
             # We don't want to process a new state if we are still in a throw.
@@ -590,7 +601,7 @@ class SlaveUnit(Unit):
         if try_next_position:
             self.offset = 0
             if self._next_position:
-                p, self.direction, self.state = self._next_position
+                p, self.direction, self.state, self.holding = self._next_position
                 if (self.x, self.y) == p:
                     self.moving = False
                 else:
@@ -606,10 +617,13 @@ class SlaveUnit(Unit):
                     self.moving = False
                     if self.snowballs == 0:
                         self._next_position = ((self.x, self.y), self.direction,
-                                "getting_snowball")
+                                "getting_snowball", None)
                 self.counter = data.get_ticks()
             else:
                 self.moving = False
+        endxy = self.x, self.y
+        if startxy != endxy:
+            self.move_sprites(startxy, endxy)
 
 
 class MasterIgloo(Igloo):
@@ -629,6 +643,12 @@ class MasterIgloo(Igloo):
             send(MBuildingJeopardy(self.building_id, j))
     jeopardy = property((lambda self:self._jeopardy), _set_jeopardy)
 
+    def storage_changed(self):
+        Igloo.storage_changed(self)
+        m = MBuildingStorage(self.building_id, self.num_storage('fish'),
+                self.num_storage('crystal'))
+        send(m)
+
 class SlaveIgloo(Igloo):
 
     def handle_MBuildingOwner(self, m):
@@ -642,6 +662,12 @@ class SlaveIgloo(Igloo):
     def handle_MBuildingJeopardy(self, m):
         self.jeopardy = bool(m.jeopardy)
 
+    def handle_MBuildingStorage(self, m):
+        self._storage['fish'] = m.fish
+        self._storage['crystal'] = m.crystal
+        self.storage_changed()
+
+
 
 from plugin import Emblem, DynamicMapObject, BaseDynamicMapObject
 
