@@ -1,9 +1,16 @@
 #!/usr/bin/env python
 from __future__ import division
+import sys
+
+try:
+    import rabbyt
+except ImportError:
+    print "Could not find the rabbyt library! You can get it at the python cheese shop.\nTry running:\n\n$ sudo easy_install rabbyt"
+    sys.exit()
+
 
 from player import Player
 
-import sys
 import struct
 
 import threading, socket
@@ -41,14 +48,19 @@ class Main:
         # For networked games a peer will be assigned here.
         self.net = None
 
+        self.load_lobby_data()
+
         unit.set_unit_base(unit.Unit)
 
     def lobby(self, mapname):
         self.mapname = mapname
+        self.load_lobby_data()
         self.running = True
         self.loading_screen()
-        self.load_data(self.mapname)
+        if data.THIS_IS_SERVER:
+            self.load_data(self.mapname)
         self.clock = pygame.time.Clock()
+        self.last_fps = 0
         self.lobbybg = textures.Texture.get("data/lobbybg.png")
         if data.THIS_IS_SERVER:
             # Create a local player on the server.
@@ -120,7 +132,7 @@ class Main:
         glVertex2i(0, vh)
         glEnd()
 
-        self.lobbybg.render(y=60)
+        self.lobbybg.render(y=60, invert_y=False)
 
         # Draw chat.
         data.messages.drawall()
@@ -299,16 +311,17 @@ class Main:
         pygame.display.flip()
         self.loading_screen_alpha = 1
 
-
-    def load_data(self, mapname):
-        data.init(mapname)
+    def load_lobby_data(self):
         self.snow = textures.Texture("data/snow.png")
         self.warningtexture = textures.Texture("data/warning.png", scale_format="linear")
         self.mouse_cursor = textures.Texture("data/mouse_cursor.png")
-        #self.brpanel = textures.Texture("data/br.png")
+
+    def load_data(self, mapname):
+        data.init(mapname)
 
         import terrain
-        self.terrain = terrain.Terrain(None)
+        self.snow_terrain = terrain.Terrain(None, "all")
+        self.ice_terrain = terrain.Terrain(None, "ice")
 
         if settings.get_option("cursor") == "1":
             pygame.mouse.set_visible(False)
@@ -340,6 +353,10 @@ class Main:
 
 
     def loop(self):
+        start = pygame.time.get_ticks()
+        #if pygame.time.get_ticks() - self.last_fps > 1000:
+            #print "FPS: ", self.clock.get_fps()
+            #self.last_fps = pygame.time.get_ticks()
         dt = self.clock.tick(35)
         if dt > 144: dt = 144
         if not ctrl.handle_events(dt):
@@ -347,6 +364,9 @@ class Main:
         else:
             display.draw(self, dt)
 
+        #end = pygame.time.get_ticks()
+        #if end-start > 60: print "stutter!", end-start
+
 
 class ServerMain(Main, networking.NetLoopBase):
     def __init__(self):
@@ -393,6 +413,8 @@ class ServerMain(Main, networking.NetLoopBase):
         # connections.  Lets call the original now...
         networking.Server.new_connection(self.net, con)
 
+        con.send(networking.MMapname(self.mapname))
+
 
     def handle_MNewPlayer(self, m):
         # A client is requesting creating a new player.
@@ -463,6 +485,7 @@ class ServerMain(Main, networking.NetLoopBase):
 
 class ClientMain(Main, NetLoopBase):
     def __init__(self, host = "localhost"):
+        data.THIS_IS_SERVER = False
         self.host = host
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         s.connect((self.host, 5555))
@@ -511,6 +534,9 @@ class ClientMain(Main, NetLoopBase):
                 getattr(player, name)(m)
         return handled
 
+    def handle_MResourceQuantity(self, m):
+        data.map.tiles[(m.tx, m.ty)].resource.quantity = m.q
+
     def handle_MNewPlayer(self, m):
         p = Player(m.name, "human", m.color)
         p.player_id = m.player_id
@@ -551,8 +577,12 @@ class ClientMain(Main, NetLoopBase):
             to_player = data.players[m.to_id]
         data.messages.display(m.msg, from_player, to_player)
 
+    def handle_MMapname(self, m):
+        self.mapname = m.mapname
+        self.load_data(m.mapname)
+
     def load_data(self, mapname):
-        data.THIS_IS_SERVER = False
+
         Main.load_data(self, mapname)
 
 
@@ -563,6 +593,7 @@ else:
     flags = OPENGL | DOUBLEBUF
 vw,vh = viewport = settings.get_res()
 pygame.display.set_mode(viewport, flags)
+pygame.display.set_caption("Snowballz")
 
 glMatrixMode(GL_PROJECTION)
 glLoadIdentity()
@@ -594,8 +625,10 @@ while True:
             arg = sys.argv[sys.argv.index("--client")+1]
         elif "--single" in sys.argv:
             gametype = "single"
-            if len(sys.argv) > 2:
-                mapname = sys.argv[2]
+        else:
+            raise ValueError("No gameplay type")
+        if len(sys.argv) > 2:
+            mapname = sys.argv[2]
 
     #glHint(GL_TEXTURE_COMPRESSION_HINT, GL_FASTEST)
 
