@@ -8,6 +8,21 @@ import snowball
 import events
 import bisect
 
+import rabbyt
+
+
+class UnitRendered(data.Rendered):
+    def __init__(self, unit):
+        self.unit = unit
+        self.x, self.y = unit.penguin_sprite.xy
+        self.height = 32
+        self.z = 1
+
+    def render(self):
+        self.unit.penguin_sprite.render()
+        if self.unit.outfit_sprite.texture_id != 0:
+            self.unit.outfit_sprite.render()
+
 
 class Unit(object):
     max_warmth = 10
@@ -19,6 +34,7 @@ class Unit(object):
         self.direction = None
 
         self.x,self.y = pos
+        self.draw_x = self.draw_y = 0
 
         self.current_frame = randint(0, 10)
         self.last_tick = 0
@@ -40,6 +56,8 @@ class Unit(object):
             self.go_to_building = igloo
             igloo.units_in_building.add(self)
 
+        self.holding = None
+
         self.targeted_unit = None
 
 
@@ -65,6 +83,12 @@ class Unit(object):
         self.plugins = set()
         self._emblems = set()
 
+
+        xy = self.x*32, self.y*32 - data.map.tiles[(self.x,
+                self.y)].elevation*data.map.elevation_multiplier
+        self.penguin_sprite = rabbyt.Sprite(shape=(0,0,32,32), xy=xy)
+        self.outfit_sprite = rabbyt.Sprite(shape=(0,0,32,32), xy=xy)
+
     def add_emblem(self, emblem):
         self._emblems.add(emblem)
 
@@ -78,31 +102,29 @@ class Unit(object):
 
 
     def draw(self):
-        x = self.x*32
-        y = self.y*32
-
-        offset = self.offset
-        if self.moving:
-            offset -= 32
-
-        x += self.direction[0]*offset
-        y += self.direction[1]*offset
+        x,y = self.penguin_sprite.xy
         self.draw_x = x
         self.draw_y = y
 
+        #self.penguin_sprite.xy = x,y
+        #self.outfit_sprite.xy = x,y
+
         if self.igloo.player: color = self.igloo.player.glcolor
         else: color = (1,1,1)
-        
+
+        self.outfit_sprite.rgb = color
+
         if data.view.zoom < 0.5:
             # So zoomed out... just draw a box.
             scolor = list(color)
             alpha = min(1-(data.view.zoom-0.2)/0.2, 1)
             scolor.append(alpha)
             glColor4f(*scolor)
+            glDisable(GL_TEXTURE_2D)
             data.draw_box(x,y,32,32)
             if alpha == 1:
-                return 
-        
+                return
+
         if self.state == "frozen":
             t = data.penguins["frozen"]
             to = None
@@ -132,19 +154,21 @@ class Unit(object):
                 t = data.penguins["walk"].images[("idle", self.direction)]
                 to = data.penguins[self.type+"walk"].images[("idle", self.direction)]
 
+        self.penguin_sprite.texture_id = t.texture_id
+        self.penguin_sprite.tex_shape = t.get_shape_tex(invert_y=True)
+        if to:
+            self.outfit_sprite.texture_id = to.texture_id
+            self.outfit_sprite.tex_shape = to.get_shape_tex(invert_y=True)
+        else:
+            self.outfit_sprite.texture_id = 0
+
         if self.selected:
             # Draw selection.
             bisect.insort(data.draw_buffer, data.Rendered(data.penguinselection,
                 x, y, z=1))
 
         # Draw penguin
-        bisect.insort(data.draw_buffer, data.Rendered(t, self.draw_x,
-                self.draw_y, z=1))
-
-        if to:
-            # Draw penguin outfit.
-            bisect.insort(data.draw_buffer, data.Rendered(to, self.draw_x,
-                    self.draw_y, z=1, colorize=color))
+        bisect.insort(data.draw_buffer, UnitRendered(self))
 
 
         for e in self._emblems:
@@ -153,7 +177,8 @@ class Unit(object):
 
         # Warmth bar.
         if self.state != "frozen" and self.warmth < self.max_warmth:
-            r = lambda w: max(min(abs((w/float(self.max_warmth)*2)-2), 1), 0)
+            glDisable(GL_TEXTURE_2D)
+            b = lambda w: max(min(abs((w/float(self.max_warmth)*2)-2), 1), 0)
             g = lambda w: max(min(w/float(self.max_warmth)*2, 1), 0)
 
             x += 32
@@ -162,7 +187,7 @@ class Unit(object):
             height = self.warmth*20/self.max_warmth
 
             glPushMatrix()
-            glColor4f(r(self.warmth),g(self.warmth),0, 1)
+            glColor4f(0,g(self.warmth),b(self.warmth), 1)
 
             glTranslatef(x, y-height, 0)
             glBegin(GL_QUADS)
@@ -193,21 +218,13 @@ class Unit(object):
                     # Finished unloading.
                     self.state = "stray"
                     if data.THIS_IS_SERVER:
-                        self.igloo.storage[self.holding] += 1
+                        self.igloo.add_storage(self.holding)
                         events.fire(events.PENGUIN_UNLOADED, self, self.holding)
                     self.holding = None
 
-            if self.moving and (self.current_frame == 6 or\
+            if self.moving and (self.current_frame == 6 or
                     self.current_frame == 9) and data.footprints_settings:
-                x = self.x*32
-                y = self.y*32
-
-                offset = self.offset
-                if self.moving:
-                    offset -= 32
-
-                x += self.direction[0]*offset
-                y += self.direction[1]*offset
+                x,y = self.penguin_sprite.xy
 
                 if self.direction[0] == 0:
                     if self.current_frame == 6:
@@ -281,6 +298,7 @@ class Unit(object):
 
 
     def move(self):
+        startxy = self.x, self.y
         # Skip the check if the penguin is moving.
         if not self.moving:
             # Check with the AI to see if we want to move somewhere.
@@ -295,6 +313,25 @@ class Unit(object):
                 self.offset = 0
                 self.check_direction()
 
+        endxy = self.x, self.y
+        if startxy != endxy:
+            self.move_sprites(startxy, endxy)
+
+
+    def move_sprites(self, startxy, endxy):
+        sx = startxy[0]*32
+        sy = startxy[1]*32 - data.map.tiles[startxy].elevation *\
+                data.map.elevation_multiplier
+        ex = endxy[0]*32
+        ey = endxy[1]*32 - data.map.tiles[endxy].elevation *\
+                data.map.elevation_multiplier
+
+        x = rabbyt.lerp(sx, ex, dt=self.walk_speed)
+        y = rabbyt.lerp(sy, ey, dt=self.walk_speed)
+
+        self.penguin_sprite.xy = x,y
+        self.outfit_sprite.xy = x,y
+
     def check_direction(self):
         direction = self.find_path()
         if direction:
@@ -602,7 +639,8 @@ class Unit(object):
         return units_in_range
 
 
-    def detect_hit(self, snowball, x, y):
+    def detect_hit(self, snowball):
+        x,y = snowball.end_tile.x, snowball.end_tile.y
         d = 0
         hit = 0
         if x == self.x and y == self.y:
@@ -617,7 +655,7 @@ class Unit(object):
             events.fire(events.PENGUIN_HIT, self, d)
 
         if hit and self.warmth > 0 and self.state != "inbuilding":
-            x,y = snowball.start_x//32, snowball.start_y//32
+            x,y = snowball.start_tile.x, snowball.start_tile.y
             self.igloo.player.alert_tiles[(x, y)] = 3
             self.warmth -= d
             self.last_tiles = []
@@ -843,7 +881,7 @@ class Worker(Unit):
                 resources = data.resources
             for r in resources:
                 if r.type == resource_type and r.quantity > 0:
-                    dist = ((r.x-self.x)**2 + (r.y-self.y)**2)**.5
+                    dist = ((r.tile.x-self.x)**2 + (r.tile.y-self.y)**2)**.5
                     if not best_option:
                         best_option = (dist,r)
                     else:
